package betterwithmods.common.registry.block.recipe;

import betterwithmods.util.InvUtils;
import net.minecraft.block.state.IBlockState;
import net.minecraft.item.ItemStack;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import org.apache.commons.lang3.ArrayUtils;

import java.util.List;
import java.util.Random;
import java.util.stream.Collectors;

/**
 * Purpose:
 *
 * @author primetoxinz
 * @version 03/19/2018
 */
public abstract class BlockRecipe {
    private final BlockIngredient input;
    private final NonNullList<ItemStack> outputs;

    public BlockRecipe(BlockIngredient input, List<ItemStack> outputs) {
        this.input = input;
        this.outputs = outputs == null ? NonNullList.func_191196_a() : InvUtils.asNonnullList(outputs.stream().filter(s -> !s.func_190926_b()).collect(Collectors.toList()));
    }

    public abstract boolean craftRecipe(World world, BlockPos pos, Random rand, IBlockState state);

    public NonNullList<ItemStack> onCraft(World world, BlockPos pos) {
        NonNullList<ItemStack> items = NonNullList.func_191196_a();
        if (consumeIngredients(world, pos)) {
            items.addAll(getOutputs());
        }
        return items;
    }

    public boolean consumeIngredients(World world, BlockPos pos) {
        return world.func_175698_g(pos);
    }

    public BlockIngredient getInput() {
        return input;
    }

    public NonNullList<ItemStack> getOutputs() {
        return outputs;
    }

    @Override
    public String toString() {
        return String.format("%s-> %s", input, getOutputs());
    }

    public boolean isInvalid() {
        return (input.isSimple() && ArrayUtils.isEmpty(input.func_193365_a())) || (outputs == null || outputs.isEmpty());
    }

    public boolean matches(World world, BlockPos pos, IBlockState state) {
        return getInput().apply(world, pos, state);
    }

    public boolean isHidden() {
        return false;
    }
}
